@@ -43,6 +43,7 @@ struct _GlJournalEntry
   gchar *gid;
   gchar *systemd_unit;
   gchar *executable_path;
+  gboolean display_time_label;
 };
 
 G_DEFINE_TYPE (GlJournalEntry, gl_journal_entry, G_TYPE_OBJECT);
@@ -58,6 +59,57 @@ typedef struct
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlJournal, gl_journal, G_TYPE_OBJECT)
 
+static void
+gl_journal_update_latest_timestamp (GlJournal *journal)
+{
+    GlJournalPrivate *priv;
+    GlJournalBootID *boot_id;
+    gint r;
+    guint64 realtime;
+
+    g_return_if_fail (GL_JOURNAL (journal));
+
+    priv = gl_journal_get_instance_private (journal);
+
+    r = sd_journal_seek_tail (priv->journal);
+    if (r < 0)
+    {
+        g_warning ("Error seeking to the end of the journal: %s",
+                   g_strerror (-r));
+    }
+
+    r = sd_journal_previous (priv->journal);
+    if (r < 0)
+    {
+        g_warning ("Error retreating the read pointer in the journal: %s",
+                   g_strerror (-r));
+    }
+
+    boot_id = &g_array_index (priv->boot_ids, GlJournalBootID,
+                              priv->boot_ids->len - 1);
+    r = sd_journal_get_realtime_usec (priv->journal,
+                                      &realtime);
+    if (r < 0)
+    {
+        g_warning ("Error retrieving the sender timestamps: %s",
+                   g_strerror (-r));
+    }
+
+    if (realtime > boot_id->realtime_last)
+    {
+        boot_id->realtime_last = realtime;
+    }
+
+    /* We set the reading cursor to the latest log entry using
+     * sd_journal_previous. Reset it here. */
+    r = sd_journal_seek_tail (priv->journal);
+    if (r < 0)
+    {
+        g_warning ("Error seeking to the end of the journal: %s",
+                   g_strerror (-r));
+    }
+}
+
 GQuark
 gl_journal_error_quark (void)
 {
@@ -72,6 +124,7 @@ gl_journal_get_boot_time (GlJournal *journal,
     gchar *time;
     gint i;
 
+    gl_journal_update_latest_timestamp (journal);
     boot_ids = gl_journal_get_boot_ids (journal);
 
     for (i = 0; i < boot_ids->len; i++)
@@ -522,6 +575,8 @@ _gl_journal_query_entry (GlJournal *self)
         g_clear_error (&error);
     }
 
+    entry->display_time_label = TRUE;
+
     return entry;
 
 out:
@@ -816,3 +871,19 @@ gl_journal_entry_get_executable_path (GlJournalEntry *entry)
 
   return entry->executable_path;
 }
+
+gboolean
+gl_journal_entry_get_display_time_label (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), FALSE);
+
+  return entry->display_time_label;
+}
+
+void
+gl_journal_entry_set_display_time_label (GlJournalEntry *entry, gboolean value)
+{
+  g_return_if_fail (GL_IS_JOURNAL_ENTRY (entry));
+
+  entry->display_time_label = value;
+}
